<dl>
  <dt>pid:</dt> <dd>{{pid}}</dd> <br>
  <dt>created:</dt> <dd>{{time_created_fmt}}</dd> <br>
  <dt>basepath:</dt> <dd>{{basepath}}</dd> <br>
  <br>
  <dt>hostname:</td> <dd>{{hostname}}</dd><br>
  <dt>interface:</td> <dd>{{interface}}</dd><br>
  <dt>port:</td> <dd>{{port}}</dd><br>
  <br>
  <dt>RSS:</dt> <dd>{{rss}}</dd> <br>
  <dt>VSZ:</dt> <dd>{{vsz}}</dd> <br>
</dl>

{{#zygotes}}
<div class="vgap">
  <h2>Zygote {{pid}}</h2>
  <dl>
	<dt>created:</dt> <dd>{{time_created_fmt}}</dd> <br>
	<dt>basepath:</dt> <dd>{{basepath}}</dd> <br>
	<br>
	<dt>RSS:</dt> <dd>{{rss}}</dd> <br>
	<dt>VSZ:</dt> <dd>{{vsz}}</dd> <br>
  </dl>
  <table class="vgap">
	<tr>
	  <th title="The PID of the worker process">PID</th>
	  <th title="When the worker process was instantiated">Time Created</th>
	  <th title="The virtual memory size (in MB) of the process">VSZ</th>
	  <th title="The resident set size (in MB) of the process">RSS</th>
	  <th title="The number of HTTP requests serviced" class="right">Reqs</th>
	  <th title="How long the request has been running for, in milliseconds">&#x394;t ms</th>
	  <th title="The remote IP of the client making the current HTTP request">Remote IP</th>
	  <th title="The full first line of the HTTP request sent by the client">HTTP Line</th>
	</tr>
	{{#workers}}
	<tr>
	  <td class="right">{{pid}}</td>
	  <td>{{time_created_fmt}}</td>
	  <td class="right">{{vsz}}</td>
	  <td class="right">{{rss}}</td>
	  <td class="right">{{request_count}}</td>
	  <td class="right">{{elapsed_formatted}}</td>
	  <td>{{remote_ip}}</td>
	  <td>{{http}}</td>
	</tr>
	{{/workers}}
  </table>
</div>
{{/zygotes}}
